@@ -34,8 +34,6 @@
 #include <libart_lgpl/libart.h>
 #include <glade/glade-xml.h>
 
-#include "egg-screen-help.h"
-
 #define FISH_APPLET(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), \
 			fish_applet_get_type(),          \
 			FishApplet))
@@ -113,7 +111,7 @@ show_help (FishApplet *fish, const char *link_id)
 {
 	GError *error = NULL;
 
-	egg_help_display_desktop_on_screen (
+	gnome_help_display_desktop_on_screen (
 		NULL, "fish-applet-2", "fish-applet-2", link_id,
 		gtk_widget_get_screen (GTK_WIDGET (fish)),
 		&error);
@@ -181,8 +179,11 @@ command_value_changed (GtkEntry   *entry,
 
 	text = gtk_entry_get_text (entry);
 
-	if (!text || !text [0])
+	if (!text || !text [0]) {
+		panel_applet_gconf_set_string (PANEL_APPLET (fish), 
+					       "command", "", NULL);
 		return;
+	}
 
 	if (!strncmp (text, "ps ", 3)  ||
 	    !strcmp  (text, "ps")      ||
@@ -462,6 +463,11 @@ display_about_dialog (BonoboUIComponent *uic,
 				     "If anyone is found using it, he "
 				     "should be promptly sent for a psychiatric "
 				     "evaluation.");
+	const char  *documenters [] = {
+		"Telsa Gwynne <hobbit@aloss.ukuu.org.uk>",
+		"Sun GNOME Documentation Team <gdocteam@sun.com>",
+          	NULL
+	};
 	char        *authors [3];
 	GdkPixbuf   *pixbuf;
 	GError      *error = NULL;
@@ -497,7 +503,7 @@ display_about_dialog (BonoboUIComponent *uic,
 				 "Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.",
 				 descr,
 				 (const char **) authors,
-				 NULL,
+				 documenters,
 				 NULL,
 				 pixbuf);
 
@@ -505,7 +511,7 @@ display_about_dialog (BonoboUIComponent *uic,
 	g_free (authors [0]);
 
 	if (pixbuf)
-		gdk_pixbuf_unref (pixbuf);
+		g_object_unref (pixbuf);
 
 	gtk_window_set_wmclass (
 		GTK_WINDOW (fish->about_dialog), "fish", "Fish");
@@ -624,6 +630,8 @@ update_fortune_dialog (FishApplet *fish)
 	g_free (text);
 
 	g_free (label_text);
+
+	set_ally_name_desc (fish->fortune_view, fish);
 }
 
 static void
@@ -676,6 +684,9 @@ display_fortune_dialog (FishApplet *fish)
 				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 				NULL);
 
+		gtk_dialog_set_has_separator (
+			GTK_DIALOG (fish->fortune_dialog), FALSE);
+
 		gtk_dialog_set_default_response (
 			GTK_DIALOG (fish->fortune_dialog), GTK_RESPONSE_CLOSE);
 
@@ -731,7 +742,6 @@ display_fortune_dialog (FishApplet *fish)
 		gtk_widget_show_all (fish->fortune_dialog);
 	}
 
-	set_ally_name_desc (fish->fortune_view, fish);
 	gtk_window_set_screen (GTK_WINDOW (fish->fortune_dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (fish)));
 	gtk_window_present (GTK_WINDOW (fish->fortune_dialog));
@@ -794,6 +804,7 @@ name_changed_notify (GConfClient *client,
 
 	update_fortune_dialog (fish);
 	set_tooltip (fish);
+	set_ally_name_desc (GTK_WIDGET (fish), fish);
 
 	if (fish->name_entry &&
 	    strcmp (gtk_entry_get_text (GTK_ENTRY (fish->name_entry)), fish->name))
@@ -939,7 +950,7 @@ init_fools_day (void)
 	fools_hour_end   = 12; /* Apparently jokes should stop at midday */
 	
 	for (i = 0; spanish_timezones [i]; i++)
-		if (!g_strcasecmp (spanish_timezones [i], location)) {
+		if (!g_ascii_strcasecmp (spanish_timezones [i], location)) {
 			/* Hah!, We are in Spain or Mexico
 			 * Spanish fool's day is 28th December
 			 */
@@ -1304,12 +1315,12 @@ fish_applet_expose_event (GtkWidget      *widget,
 	else
 		src_x += ((width * fish->current_frame) / fish->n_frames);
 
-	gdk_draw_pixmap (widget->window,
-			 widget->style->fg_gc [GTK_WIDGET_STATE (widget)],
-			 fish->pixmap,
-			 src_x, src_y,
-			 event->area.x, event->area.y,
-			 event->area.width, event->area.height);
+	gdk_draw_drawable (widget->window,
+			   widget->style->fg_gc [GTK_WIDGET_STATE (widget)],
+			   fish->pixmap,
+			   src_x, src_y,
+			   event->area.x, event->area.y,
+			   event->area.width, event->area.height);
 
         return FALSE;
 }
@@ -1488,6 +1499,7 @@ setup_fish_widget (FishApplet *fish)
 	setup_timeout (fish);
 
 	set_tooltip (fish);
+	set_ally_name_desc (GTK_WIDGET (fish), fish);
 
 	g_signal_connect (fish, "key_press_event",
 			  G_CALLBACK (handle_keypress), fish);
