@@ -39,6 +39,7 @@
 #include "panel-a11y.h"
 #include "panel-globals.h"
 #include "panel-multiscreen.h"
+#include "panel-lockdown.h"
 
 #include "quick-desktop-reader.h"
 
@@ -65,6 +66,43 @@ launcher_get_screen (Launcher *launcher)
 	return gtk_window_get_screen (GTK_WINDOW (panel_widget->toplevel));
 }
 
+static void
+launcher_widget_open_dialog_destroyed (GtkWidget *dialog,
+				       Launcher *launcher)
+{
+	g_return_if_fail (launcher->error_dialogs != NULL);
+
+	launcher->error_dialogs = g_slist_remove (launcher->error_dialogs, dialog);
+}
+                                                                                                                            
+static void
+launcher_widget_destroy_open_dialogs (Launcher *launcher)
+{
+	GSList *l, *list;
+
+	list = launcher->error_dialogs;
+	launcher->error_dialogs = NULL;
+
+	for (l = list; l; l = l->next) {
+		g_signal_handlers_disconnect_by_func (G_OBJECT (l->data),
+						      G_CALLBACK (launcher_widget_open_dialog_destroyed),
+						      launcher);
+		gtk_widget_destroy (l->data);
+	}
+	g_slist_free (list);
+}
+
+static void
+launcher_register_error_dialog (Launcher *launcher, 
+				GtkWidget *dialog)
+{
+	launcher->error_dialogs = g_slist_append (launcher->error_dialogs,
+						  dialog);
+	g_signal_connect (dialog, "destroy",
+			  G_CALLBACK (launcher_widget_open_dialog_destroyed),
+			  launcher);
+}
+
 static const char *
 panel_launcher_get_filename (const char *path)
 {
@@ -87,11 +125,11 @@ panel_launcher_save_ditem (GnomeDesktopItem *ditem,
 
 	location = gnome_desktop_item_get_location (ditem);
 	if (!location) {
-		char *path;
+		char *uri;
 
-		path = panel_make_unique_path (NULL, ".desktop");
-		gnome_desktop_item_set_location (ditem, path);
-		g_free (path);
+		uri = panel_make_unique_uri (NULL, ".desktop");
+		gnome_desktop_item_set_location (ditem, uri);
+		g_free (uri);
 	}
 
 	gnome_desktop_item_save (ditem, NULL, TRUE, &error);
@@ -124,22 +162,27 @@ launch_url (Launcher *launcher)
 	screen = launcher_get_screen (launcher);
 
 	if (!url) {
-		panel_error_dialog (screen,
-				    "no_url_dialog",
-				    _("Cannot launch icon"),
-				    _("This launch icon does not specify a url to show."));
+		GtkWidget *error_dialog;
+
+		error_dialog = panel_error_dialog (screen,
+						   "no_url_dialog",
+						   _("Cannot launch icon"),
+						   _("This launch icon does not specify a url to show."));
+		launcher_register_error_dialog (launcher, error_dialog);
 		return;
 	}
 
 	gnome_url_show_on_screen (url, screen, &error);
 	if (error) {
-		panel_error_dialog (screen,
-				    "cannot_show_url_dialog",
-				    _("Cannot show %s"),
-				    "%s",
-				    url,
-				    error->message);
-
+		GtkWidget *error_dialog;
+	
+		error_dialog = panel_error_dialog (screen,
+						   "cannot_show_url_dialog",
+						   _("Cannot show %s"),
+						   "%s",
+						   url,
+						   error->message);
+		launcher_register_error_dialog (launcher, error_dialog);
 		g_clear_error (&error);
 	}
 }
@@ -167,12 +210,14 @@ launch_cb (GtkWidget *widget,
 		panel_ditem_launch (
 			item, NULL, 0, launcher_get_screen (launcher), &error);
 		if (error) {
-			panel_error_dialog (launcher_get_screen (launcher),
-					    "cannot_launch_icon",
-					    _("Cannot launch icon"),
-					    "%s",
-					    error->message);
+			GtkWidget *error_dialog;
 
+			error_dialog = panel_error_dialog (launcher_get_screen (launcher),
+							   "cannot_launch_icon",
+							   _("Cannot launch icon"),
+							   "%s",
+							   error->message);
+			launcher_register_error_dialog (launcher, error_dialog);
 			g_clear_error (&error);
 		}
 	}
@@ -211,11 +256,13 @@ drag_data_received_cb (GtkWidget        *widget,
 	g_strfreev (envp);
 
 	if (error) {
-		panel_error_dialog (launcher_get_screen (launcher),
-				    "cannot_launch_icon",
-				    _("Cannot launch icon"),
-				    "%s",
-				    error->message);
+		GtkWidget *error_dialog;
+		error_dialog = panel_error_dialog (launcher_get_screen (launcher),
+						   "cannot_launch_icon",
+						   _("Cannot launch icon"),
+						   "%s",
+						   error->message);
+		launcher_register_error_dialog (launcher, error_dialog);
 		g_clear_error (&error);
 	}
 
@@ -227,6 +274,7 @@ destroy_launcher (GtkWidget *widget,
 		  Launcher  *launcher)
 {
 	launcher_properties_destroy (launcher);
+	launcher_widget_destroy_open_dialogs (launcher);
 }
 
 void
@@ -474,6 +522,35 @@ create_launcher (const char *location)
 	return launcher;
 }
 
+static char *
+guess_icon_from_exec (GnomeDesktopItem *ditem)
+{
+	const char *exec;
+	char       *icon_name;
+	char       *icon;
+
+	exec = gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC);
+	if (!exec || !exec [0])
+		return NULL;
+
+	icon_name = g_path_get_basename (exec);
+
+	if (!(icon = gnome_desktop_item_find_icon (panel_icon_theme, icon_name, 48, 0))) {
+		g_free (icon_name);
+		return NULL;
+	}
+
+	if (g_file_test (icon, G_FILE_TEST_IS_DIR)) {
+		g_free (icon_name);
+		g_free (icon);
+		return NULL;
+	}
+
+	g_free (icon);
+
+	return icon_name;
+}
+
 static void
 setup_button (Launcher *launcher)
 {
@@ -482,6 +559,7 @@ setup_button (Launcher *launcher)
 	char *str;
 	const char *icon;
 	char *unescaped_str;
+	char *freeme = NULL;
 	
 	g_return_if_fail (launcher != NULL);
 
@@ -509,7 +587,12 @@ setup_button (Launcher *launcher)
 	/* Setup icon */
 	icon = gnome_desktop_item_get_string (launcher->ditem,
 					      GNOME_DESKTOP_ITEM_ICON);
+	if (!icon)
+		icon = freeme = guess_icon_from_exec (launcher->ditem);
+
 	button_widget_set_icon_name (BUTTON_WIDGET (launcher->button), sure_string (icon));
+
+	g_free (freeme);
 }
 
 
@@ -582,7 +665,7 @@ window_response (GtkWidget *w, int response, gpointer data)
 	if (response == GTK_RESPONSE_HELP) {
 		panel_show_help (
 			gtk_window_get_screen (GTK_WINDOW (w)),
-			"wgospanel.xml", "gospanel-52");
+			"user-guide.xml", "gospanel-52");
 	} else if (response == REVERT_BUTTON) { /* revert */
 		if (launcher->ditem != NULL)
 			gnome_desktop_item_unref (launcher->ditem);
@@ -692,9 +775,6 @@ void
 launcher_properties (Launcher  *launcher,
 		     GdkScreen *screen)
 {
-	if (panel_profile_get_inhibit_command_line ())
-		return;
-
 	if (launcher->prop_dialog != NULL) {
 		gtk_window_set_screen (
 			GTK_WINDOW (launcher->prop_dialog), screen);
@@ -706,6 +786,16 @@ launcher_properties (Launcher  *launcher,
 	gtk_widget_show_all (launcher->prop_dialog);
 }
 
+static gboolean
+lancher_properties_enabled (void)
+{
+	if (panel_lockdown_get_locked_down () ||
+	    panel_lockdown_get_disable_command_line ())
+		return FALSE;
+
+	return TRUE;
+}
+
 static Launcher *
 load_launcher_applet (const char       *location,
 		      PanelWidget      *panel,
@@ -728,15 +818,11 @@ load_launcher_applet (const char       *location,
 	if (!launcher->info)
 		return NULL;
 
-	if ( ! panel_profile_get_locked_down () &&
-	     ! panel_toplevel_get_locked_down (panel->toplevel) &&
-	     /* if the command line is inhibited, don't allow editting launchers,
-		since that would allow running arbitrary commands */
-	     ! panel_profile_get_inhibit_command_line ())
-		panel_applet_add_callback (launcher->info,
-					   "properties",
-					   GTK_STOCK_PROPERTIES,
-					   _("_Properties"));
+	panel_applet_add_callback (launcher->info,
+				   "properties",
+				   GTK_STOCK_PROPERTIES,
+				   _("_Properties"),
+				   lancher_properties_enabled);
 
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (launcher->button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (launcher->button), TRUE);
@@ -906,7 +992,7 @@ really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 	} else if (response == GTK_RESPONSE_HELP) {
 		panel_show_help (
 			gtk_window_get_screen (GTK_WINDOW (dialog)),
-			"wgospanel.xml", "gospanel-52");
+			"user-guide.xml", "gospanel-52");
 		/* just return as we don't want to close */
 		return;
 	}
@@ -1033,8 +1119,9 @@ panel_launcher_create_with_id (const char    *toplevel_id,
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "launcher_location");
 	gconf_client_set_string (client, key, location, NULL);
 
-	/* frees id */
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
+
+	g_free (id);
 }
 
 void
@@ -1057,7 +1144,7 @@ panel_launcher_create_copy (PanelToplevel *toplevel,
 	char        *new_location;
 	const char  *filename;
 
-	new_location = panel_make_unique_path (NULL, ".desktop");
+	new_location = panel_make_unique_uri (NULL, ".desktop");
 	
 	source_uri = gnome_vfs_uri_new (location);
 	dest_uri   = gnome_vfs_uri_new (new_location);
